package com.jobs.provider.controller;

import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.netflix.hystrix.contrib.javanica.annotation.HystrixProperty;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@RequestMapping("/provider")
@RestController
public class ProviderController {

    /*
    服务提供者会在两种情况下发生降级：
    1 发生异常
    2 业务逻辑处理超时（
    --- 对于 Hystrix 来说，默认必须在 1 秒内处理完业务逻辑，否则就会超时
    --- 可以针对具体的接口方法，配置 Hystrix 的业务逻辑超时时间，比如配置为 3 秒
     */

    //这个是正常方法，不会发生降级
    @RequestMapping("/test1/{id}")
    public Map GetData(@PathVariable("id") int id) {

        Map result = new HashMap();
        result.put("status", 0);
        result.put("msg", "success");
        result.put("method", "GetData");
        result.put("get_id_value", id);
        result.put("version", UUID.randomUUID().toString());

        return result;
    }

    //-----------------------------------

    //该方法会发生异常，从而调用 @HystrixCommand 方法所配置的降级方法
    @HystrixCommand(fallbackMethod = "GetErrorFallback")
    @RequestMapping("/test2/{id}")
    public Map GetError(@PathVariable("id") int id) {

        //执行此处，会发生异常，导致调用降级方法
        int num = id / 0;

        Map result = new HashMap();
        result.put("status", 0);
        result.put("msg", "success");
        result.put("data", num);
        result.put("version", UUID.randomUUID().toString());

        return result;
    }

    //定义 GetError 的降级方法
    //方法的返回值、参数列表、参数类型，要与原方法保持一致
    public Map GetErrorFallback(int id) {

        Map result = new HashMap();
        result.put("status", 1);
        result.put("msg", "fallback...");
        result.put("method", "GetError");
        result.put("get_id_value", id);
        result.put("remark", "服务发生异常，导致降级...");

        return result;
    }

    //-----------------------------------

    //该方法会发生业务逻辑处理超时，因为 Hstrix 默认超时时间是 1 秒
    //从而调用 @HystrixCommand 方法所配置的降级方法
    @HystrixCommand(fallbackMethod = "GetTimeOutFallback")
    @RequestMapping("/test3/{id}")
    public Map GetTimeOut1(@PathVariable("id") int id) {

        /*
        默认情况下，Hstrix 要求必须在 1 秒内处理完业务逻辑并返回结果
        在此休眠 2 秒钟，导致服务端接口的业务逻辑处理超时，导致调用降级方法
        注意：
            ---超时是由服务端接口造成的。
            ---如果此时对客户端进行配置，想要避免超时，属于瞎折腾，白费功夫。
        */

        try {
            Thread.sleep(2000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        Map result = new HashMap();
        result.put("status", 0);
        result.put("msg", "success");
        result.put("get_id_value", id);
        result.put("version", UUID.randomUUID().toString());

        return result;
    }

    //定义 GetTimeOut 的降级方法
    //方法的返回值、参数列表、参数类型，要与原方法保持一致
    public Map GetTimeOutFallback(int id) {

        Map result = new HashMap();
        result.put("status", 1);
        result.put("msg", "fallback...");
        result.put("method", "GetError");
        result.put("get_id_value", id);
        result.put("remark", "服务业务处理超时，导致降级...");

        return result;
    }

    //-----------------------------------

    //该方法处理业务逻辑处理，不会发生超时，
    //因为 Hstrix 配置的【处理时间】大于【实际所需要的业务逻辑处理时间】
    @HystrixCommand(fallbackMethod = "GetTimeOutFallback", commandProperties = {
            //设置 Hystrix 的允许的业务逻辑处理超时时间为 3 秒
            //想要查找可以设置的 HystrixProperty 值，可以按两下 shift 键，
            //输入 HystrixCommandProperty，然后查看 HystrixCommandProperties 类的构造方法
            @HystrixProperty(name = "execution.isolation.thread.timeoutInMilliseconds",
                    value = "3000")})
    @RequestMapping("/test4/{id}")
    public Map GetTimeOut2(@PathVariable("id") int id) {

        //这里虽然休眠 2 秒，但是设置了 Hystrix 允许业务逻辑处理时间为 3 秒，所以不会超时
        try {
            Thread.sleep(2000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        Map result = new HashMap();
        result.put("status", 0);
        result.put("msg", "success");
        result.put("get_id_value", id);
        result.put("version", UUID.randomUUID().toString());

        return result;
    }
}




